package cn.jisol.game.actions;

import cn.hutool.core.util.IdUtil;
import cn.hutool.log.LogFactory;
import cn.jisol.game.network.client.GRefereeClient;
import cn.jisol.game.proto.GPVPMessage;
import cn.jisol.ngame.actions.core.NAction;
import cn.jisol.ngame.actions.core.NActionMethod;
import cn.jisol.ngame.util.EventDispatcher;
import cn.jisol.ngame.util.JLoggerUtil;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * 裁决员行为 处理裁决
 */
@NAction
public class GRefereeAction {

    //裁决员列表
    public static Map<String, GRefereeClient> REFEREES;

    //裁决员线程列表
    public static Map<String,Thread> THREADS = new HashMap<>();

    //最大裁决员数量
    public static int MAX = 10;

    //headless位置
//    public static String Headless = "D:\\myproject\\JisolGame\\headless";
    public static String Headless = "D:\\Jisol\\JisolGame\\headless";
//    public static String Headless = "D:\\wwwroot\\pet.jisol.cn\\headless";

    //获取一个空闲的裁决员
    public static GRefereeClient getFreeReferee(){

        for (String key : REFEREES.keySet()) {
            if (REFEREES.get(key).isReady) {
                REFEREES.get(key).isReady = false;
                return REFEREES.get(key);
            }
        }

        //如果没有空闲的裁决客户端则添加
        addRefereeClient();

        return null;

    }

    //放回一个空闲裁决员
    @NActionMethod(GActionEnum.CR_REFEREE_READY)
    public static void addFreeReferee(GRefereeClient referee){
        referee.isReady = true;
        EventDispatcher.getInstance().dispatchEvent(GActionEvent.REFEREE_FREE);
    }

    //自动回收裁决
    public static boolean autoReferee(GRefereeActionAutoInter inter){
        GRefereeClient referee = getFreeReferee();
        if (Objects.nonNull(referee)) {
            referee = inter.run(referee);
            //未使用则收回
            if(Objects.nonNull(referee)){
                addFreeReferee(referee);
                return false;
            }
            return true;
        }
        return false;
    }

    //添加裁判客户端
    public static void addRefereeClient(){

        //生成客户端Id
        String uuid = IdUtil.simpleUUID();
        //创建日志类
        Logger log = JLoggerUtil.file(uuid, Headless+"\\logger", uuid + ".log");

        if(Objects.isNull(log)) return;

        log.setUseParentHandlers(false);

        Thread thread = new Thread(() -> {
            try {
                Process process = Runtime.getRuntime().exec("cmd /c npm run main", null, new File(Headless));
                InputStream is = process.getInputStream();
                InputStreamReader isr = new InputStreamReader(is);
                BufferedReader br = new BufferedReader(isr);
                String content = br.readLine();
                while (content != null) {
                    log.log(Level.ALL, content);
                    content = br.readLine();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        });

        thread.start();

        //保存线程
        THREADS.put(uuid,thread);


    }

    public interface GRefereeActionAutoInter{
        GRefereeClient run(GRefereeClient referee);
    }

}
