package cn.jisol.game.controller.argsresolver;

import cn.jisol.game.controller.exception.PlayerException;
import cn.jisol.game.controller.exception.TokenException;
import cn.jisol.game.controller.game.GPlayerController;
import cn.jisol.game.data.Cache;
import cn.jisol.game.entity.User;
import cn.jisol.game.entity.game.Player;
import cn.jisol.game.service.impl.PlayerServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import java.util.Objects;

@Component
public class PlayerMethodArgumentResolver implements HandlerMethodArgumentResolver {

    GPlayerController playerController;

    public PlayerMethodArgumentResolver(GPlayerController playerController) {
        this.playerController = playerController;
    }

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        if (parameter.hasParameterAnnotation(CurrentPlayer.class)) {
            return true;
        }
        return false;
    }

    @Override
    public Object resolveArgument(MethodParameter methodParameter, ModelAndViewContainer modelAndViewContainer, NativeWebRequest nativeWebRequest, WebDataBinderFactory webDataBinderFactory) throws Exception {

        //获取Token
        String token = nativeWebRequest.getHeader(Cache.KEY_TOKEN);
        User user;
        Player player;
        if(Objects.isNull(token) || Objects.isNull(user = Cache.TOKEN.get(token))){
            //抛出Token异常
            throw new TokenException();
        }
        //获取Player
        if(Objects.isNull(player = Cache.PLAYER.get(user.getUserId()))){
            //如果没有数据则向数据库获取
            player = playerController.getPlayerInfo(user).data;

            if(Objects.isNull(player)){
                //抛出Token异常
                throw new TokenException();
            }else{
                Cache.PLAYER.put(user.getUserId(),player);
            }
        }

        return player;

    }

}
