package cn.jisol.game.controller.game;

import cn.jisol.game.controller.HttpCode;
import cn.jisol.game.controller.argsresolver.CurrentUser;
import cn.jisol.game.entity.User;
import cn.jisol.game.entity.game.Player;
import cn.jisol.game.service.PlayerService;
import cn.jisol.game.service.impl.PlayerServiceImpl;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;

/**
 * 玩家
 */
@Api(value = "JNGameDemo - API", tags = {"PET - API"})
@RestController
@RequestMapping("/game/player")
@ResponseBody
public class GPlayerController {

    @Autowired
    PlayerServiceImpl playerService;

    //获取玩家信息
    @ApiImplicitParams({})
    @ApiOperation(value = "获取玩家信息")
    @GetMapping("/info")
    public NewsContext<Player> getPlayerInfo(@CurrentUser User user){

        Player info = playerService.getOne(Wrappers.lambdaQuery(Player.class).eq(Player::getUserId,user.getUserId()));

        if (Objects.isNull(info)){
            try{
                info = Player.builder().userId(user.getUserId()).playerName("冒险者").build();
                playerService.saveOrUpdate(info);
            }catch (Exception ignored){}
            assert playerService != null;
            info = playerService.getOne(Wrappers.lambdaQuery(Player.class).eq(Player::getUserId,user.getUserId()));
        }

        if (info.isNovice()){
            return NewsContext.onSuccess("获取成功",info);
        }else{
            //前往 新手引导
            return NewsContext.onMessage("前往新手引导",info, HttpCode.NOT_CREATE_PLAYER_INFO);
        }

    }

    //设置玩家信息
    @ApiImplicitParams({})
    @ApiOperation(value = "设置玩家信息")
    @PostMapping("/info/save")
    public NewsContext<Player> savePlayerInfo(@CurrentUser User user,@RequestBody Player player){

        //获取玩家信息
        Player info = playerService.getOne(Wrappers.lambdaQuery(Player.class).eq(Player::getUserId,user.getUserId()));

        //如果没有玩家信息则初始化信息
        if (Objects.isNull(info)){
            info = Player.builder().userId(user.getUserId()).build();
        }

        info.setPlayerName(player.getPlayerName());
        info.setNovice(player.isNovice());

        //保存信息
        if(playerService.saveOrUpdate(info)){
            return NewsContext.onSuccess("保存成功",info);
        }else{
            return NewsContext.onFail("保存失败",info);
        }

    }


}
