package cn.jisol.game.controller.game;

import cfg.TB.TbGRoleUpLevel;
import cfg.TbGEntity.TResource;
import cn.jisol.game.controller.argsresolver.CurrentPlayer;
import cn.jisol.game.data.TD;
import cn.jisol.game.entity.game.Player;
import cn.jisol.game.entity.game.PlayerPet;
import cn.jisol.game.entity.game.Resource;
import cn.jisol.game.service.PlayerPetService;
import cn.jisol.game.service.ResourceService;
import cn.jisol.game.vo.news.NewsResource;
import cn.jisol.game.vo.news.ov.ResourceUpdateOV;
import cn.jisol.game.vo.pet.PetUpStarOV;
import cn.jisol.ngame.util.NewsContext;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.retry.RetryException;
import org.springframework.retry.annotation.Retryable;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 玩家宠物
 */
@Api(value = "JNGameDemo - API", tags = {"PET - API"})
@RestController
@RequestMapping("/game/pet")
@ResponseBody
public class GPlayerPetController {

    @Autowired
    PlayerPetService playerPetService;
    @Autowired
    ResourceService resourceService;

    //获取玩家宠物列表
    @ApiImplicitParams({})
    @ApiOperation(value = "获取玩家宠物列表")
    @GetMapping("/list")
    public NewsContext<List<PlayerPet>> getPetList(@CurrentPlayer Player player){

        return NewsContext.onSuccess("获取成功",
                playerPetService.list(
                        Wrappers.lambdaQuery(PlayerPet.class).eq(PlayerPet::getPetPlayerId,player.getPlayerId())
                )
        );

    }

    //提升宠物星
    @ApiImplicitParams({})
    @ApiOperation(value = "提升宠物星")
    @PostMapping("/up/star")
    @Transactional
    public NewsResource<PlayerPet> onUpStar(@RequestBody PetUpStarOV info, @CurrentPlayer Player player){

        //判断参数
        if(info.getPets().size() <= 0 || Objects.isNull(info.getPetId())){
            return NewsResource.onFail("参数错误");
        }

        //查询被合成的宠物列表
        //合成数量
        long petCount = playerPetService.count(
                Wrappers.lambdaQuery(PlayerPet.class)
                        .eq(PlayerPet::getPetPlayerId, player.getPlayerId())
                        .and(qr -> {
                            //遍历被合成的宠物
                            info.getPets().forEach(petId -> {
                                qr.eq(PlayerPet::getPetId, petId);
                                qr.or();
                            });
                        })
        );

        if(info.getPets().size() != petCount){
            return NewsResource.onFail("合成宠物信息错误");
        }

        //查询被升级的宠物
        PlayerPet pet = playerPetService.getById(info.getPetId());

        //判断宠物是否是自己的
        if(!Objects.equals(pet.getPetPlayerId(), player.getPlayerId())){
            return NewsResource.onFail("宠物不是自己的");
        }

        //升级星
        pet.setPetStarExp(pet.getPetStarExp() + info.getPets().size());

        //判断是否可以升下一星 如果可以则 升
        int upStarExp;
        while (pet.getPetStarExp() >= (upStarExp = TD.DATA.getTbGRoleUpStar().get(pet.getPetStar() + 1).merge)){
            //消耗经验
            pet.setPetStarExp(pet.getPetStarExp() - upStarExp);
            //升星
            pet.setPetStar(pet.getPetStar() + 1);
        }

        //保存宠物信息 (保存失败则重试)
        if(!playerPetService.updateById(pet)) throw new RetryException("");

        //删除被合成的宠物
        playerPetService.removeByIds(info.getPets());

        //整理刷新数据
        ArrayList<ResourceUpdateOV> resources = new ArrayList<>();
        //刷新宠物
        resources.add(ResourceUpdateOV.onPet(ResourceUpdateOV.ResourceUpdateType.UPDATE,pet));
        //删除被合成的宠物
        info.getPets().forEach(item -> {
            resources.add(ResourceUpdateOV.onPet(ResourceUpdateOV.ResourceUpdateType.MINUS,PlayerPet.builder().petId(new Long(item)).build()));
        });

        return NewsResource.onSuccess("升星成功",null, resources.toArray(new ResourceUpdateOV[0]));

    }

    //提升宠物等级
    @ApiImplicitParams({})
    @ApiOperation(value = "提升宠物等级")
    @PostMapping("/up/level/{petId}")
    @Transactional
    public NewsResource<PlayerPet> onUpLevel(@PathVariable Integer petId, @CurrentPlayer Player player){

        //整理刷新数据
        ArrayList<ResourceUpdateOV> resources = new ArrayList<>();

        //获取需要升级的宠物
        PlayerPet pet = playerPetService.getById(petId);
        if(Objects.isNull(pet) || !Objects.equals(pet.getPetPlayerId(), player.getPlayerId())){
            return NewsResource.onFail("你未拥有这个宠物");
        }

        TbGRoleUpLevel resource = TD.DATA.getTbGRoleUpLevel().get(pet.getPetLevel() + 1);

        if(Objects.isNull(resource)) return NewsResource.onFail("无法升级");

        //星级不够则 无法升级
        if(pet.getPetStar() * 10 <= pet.getPetLevel()){
            return NewsResource.onFail("突破星级 可升级");
        }

        //扣除玩家材料
        for (int i = 0; i < resource.materials.length; i++) {
            TResource res = resource.materials[i];
            Resource data = resourceService.addResourceValue(player.getPlayerId(), res.id, -((long) res.value));
            if (Objects.isNull(data)){
                return NewsResource.onFail("材料不够");
            }else{
                //刷新资源
                resources.add(ResourceUpdateOV.onUpdate(data));  //刷新资源
            }
        }

        //升级
        pet.setPetLevel(pet.getPetLevel() + 1);

        //保存
        if(!playerPetService.updateById(pet)) throw new RetryException("");
        resources.add(ResourceUpdateOV.onPet(ResourceUpdateOV.ResourceUpdateType.UPDATE,pet));  //刷新宠物

        return NewsResource.onSuccess("升级成功",pet,resources.toArray(new ResourceUpdateOV[0]));

    }

}
